package be.bitscuit.mike;

import androidx.appcompat.app.AppCompatActivity;

import android.media.AudioAttributes;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioRecord;
import android.media.AudioTrack;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.os.Bundle;
import android.widget.CompoundButton;
import android.widget.Switch;
import android.widget.TextView;

import static android.os.Process.setThreadPriority;

public class MainActivity extends AppCompatActivity {

    // Views
    private TextView textState;
    private Switch switchOnOff, switchMute, switchStandby;

    // State vars
    private boolean recording = false;
    private boolean muted = false;
    private boolean standby = false;

    // Sound vars
    private MediaPlayer soundStandby;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Init views
        textState = findViewById(R.id.main_text_state);
        switchOnOff = findViewById(R.id.main_switch_on_off);
        switchMute = findViewById(R.id.main_switch_mute);
        switchStandby = findViewById(R.id.main_switch_standby);

        // Set view props
        switchOnOff.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(b){
                    // When switch on
                    recordStart();
                }else{
                    // When switch off
                    recordStop();
                }
            }
        });
        switchMute.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(b){
                    // When switch on
                    muted = true;
                    updateUI();
                }else{
                    // When switch off
                    muted = false;
                    updateUI();
                }
            }
        });
        switchStandby.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(b){
                    // When switch on
                    standby = true;

                    // Start sound
                    soundStandby.seekTo(0);
                    soundStandby.start();

                    // Update UI
                    updateUI();
                }else{
                    // When switch off
                    standby = false;

                    // Stop sound
                    soundStandby.pause();

                    // Update UI
                    updateUI();
                }
            }
        });

        // Load sounds
        soundStandby = MediaPlayer.create(this, R.raw.sound_standby);
        soundStandby.setLooping(true);
        soundStandby.setVolume(0.05f, 0.05f);
    }


    private void recordStart(){
        if(recording) return; // Return if already recording

        // Start recording
        recording = true;

        // Start thread
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {

                // Init recorder
                int minsize = AudioRecord.getMinBufferSize(
                        44100,
                        AudioFormat.CHANNEL_IN_MONO,
                        AudioFormat.ENCODING_PCM_16BIT
                );
                AudioRecord audioRecord = new AudioRecord(
                        MediaRecorder.AudioSource.MIC,
                        44100,
                        AudioFormat.CHANNEL_IN_MONO,
                        AudioFormat.ENCODING_PCM_16BIT,
                        minsize
                );

                // Init player
                setThreadPriority(android.os.Process.THREAD_PRIORITY_URGENT_AUDIO);
                AudioTrack audioTrack = new AudioTrack(
                        AudioManager.STREAM_MUSIC,
                        44100,
                        AudioFormat.CHANNEL_OUT_MONO,
                        AudioFormat.ENCODING_PCM_16BIT,
                        minsize,
                        AudioTrack.MODE_STREAM
                );

                // Start
                audioRecord.startRecording();
                audioTrack.play();

                // Loop
                while(recording){

                    // Get new data
                    short[] buffer = new short[minsize];
                    audioRecord.read(buffer, 0, minsize);

                    // Manipulate data

                    // Mute
                    if(muted || standby){
                        // Empty buffer
                        buffer = new short[minsize];

                        for(int i=0; i<minsize; i++){
                            buffer[i] = 0;
                        }
                    }

                    // Write data to output
                    audioTrack.write(buffer, 0, minsize);
                }

                // Stop recording
                audioRecord.stop();
                audioTrack.stop();

            }
        });
        thread.start();

        // Change UI
        updateUI();
    }

    private void recordStop(){
        if(!recording) return; // Return if not recording

        // Stop recording
        recording = false;

        // Change UI
        updateUI();
    }




    private void updateUI(){

        // Set right state text
        String stateText = "IDLE";
        if(recording){
            stateText = "ON AIR";

            if(muted) stateText = "MUTED";
            if(standby) stateText = "STAND BY";
        }

        textState.setText(stateText);

    }
}